using System;
using System.Drawing;
using ArrayList=System.Collections.ArrayList;
using ImageFormat=System.Drawing.Imaging.ImageFormat;

namespace TESFaith {
    public class MapDrawer {
        private const int SquareSizeL=12;
        private const int SquareSizeM=SquareSizeL/2;
        private const int SquareSizeS=SquareSizeL/4;

        private const int OutSizeL=16;
        private const int OutSizeM=OutSizeL/2;
        private const int OutSizeS=OutSizeL/4;
        private static Brush OverlayBrush=new SolidBrush(Color.FromArgb(127,127,127,0));

        private static ArrayList OverlaidCells=new ArrayList();
        private static ArrayList OverlayNames=new ArrayList();
        private static ArrayList CurrentCells;
        private static int LeftCorner=-30;
        private static int TopCorner=30;
        private static int xSize=40;
        private static int ySize=40;
        public static System.Windows.Forms.Panel panel;
        private static int CurrentCellX=-30000;
        private static int CurrentCellY=-30000;
        public static int SelectedX,SelectedY;
        private static int Overlays=0;
        private static bool selected;
        public static bool Selected {
            get { return selected; }
            set { selected=value; panel.Invalidate(); }
        }

        public static void GetCellPos(int x,int y,out int xpos,out int ypos) {
            Point p=new Point(x,y);
            xpos=LeftCorner+(p.X/SquareSizeL);
            ypos=TopCorner-(p.Y/SquareSizeL);
        }

        public static bool CellExists(int xpos,int ypos) {
            foreach(ChangedCell cc in CurrentCells) {
                if(cc.OldX==xpos&&cc.OldY==ypos) {
                    return true;
                }
            }
            return false;
        }

        public static void Scroll(int x,int y) {
            LeftCorner+=x;
            TopCorner+=y;
            panel.Invalidate();
        }

        public static void UpdateSize(object o,EventArgs e) {
            int oldx=xSize;
            int oldy=ySize;
            xSize=(panel.Width/SquareSizeL)+1;
            ySize=(panel.Height/SquareSizeL)+1;
            if(xSize>oldx||ySize>oldy) panel.Invalidate();
        }

        public static void AddOverlay(ArrayList list,string plugin) {
            for(int i=0;i<list.Count;i++) {
                ChangedCell cc=(ChangedCell)list[i];
                cc.Plugin=plugin;
                cc.Overlay=Overlays;
                OverlaidCells.Add(cc);
            }
            Overlays++;
            OverlayNames.Add(plugin);
            panel.Invalidate();
        }

        public static void GetMousePos(int x,int y,out string message) {
            Point p=new Point(x,y);
            int xpos=LeftCorner+(p.X/SquareSizeL);
            int ypos=TopCorner-(p.Y/SquareSizeL);
            if(xpos!=CurrentCellX||ypos!=CurrentCellY) {
                CurrentCellX=xpos;
                CurrentCellY=ypos;
                message="("+xpos.ToString()+", "+ypos.ToString()+")";
                if(CurrentCells!=null) {
                    foreach(ChangedCell cc in CurrentCells) {
                        if(cc.OldX==xpos&&cc.OldY==ypos) {
                            message+="\n[ACTIVE] ";
                            if(cc.Name!="") message+="\""+cc.Name+"\", ";
                            else message+="Cell has no name, ";
                            if(cc.RegionName!="") message+="\""+cc.RegionName+"\"";
                            else message+="Cell has no associated region";
                            break;
                        }
                    }
                }
                foreach(ChangedCell cc in OverlaidCells) {
                    if(cc.OldX==xpos&&cc.OldY==ypos) {
                        message+="\n["+cc.Plugin+"] ";
                        if(cc.Name!="") message+="\""+cc.Name+"\", ";
                        else message+="Cell has no name, ";
                        if(cc.RegionName!="") message+="\""+cc.RegionName+"\"";
                        else message+="Cell has no associated region";
                    }
                }
            } else {
                message=null;
            }
        }

        public static void ClearOverlay() {
            Overlays=0;
            OverlaidCells.Clear();
            OverlayNames.Clear();
            panel.Invalidate();
        }

        public static void SetActiveCells(ArrayList list) {
            if(list==null) {
                CurrentCells.Clear();
                CurrentCells=null;
            } else {
                CurrentCells=new ArrayList();
                CurrentCells.AddRange(list);
            }
            panel.Invalidate();
        }

        public static void PaintMap(System.Windows.Forms.PaintEventArgs e) {
            //Possibly not the most efficient way of doing things, but it works well enough
            e.Graphics.Clear(Color.White);
            
            foreach(ChangedCell cc in OverlaidCells) {
                if(cc.OldX<LeftCorner||cc.OldX>=LeftCorner+xSize) continue;
                if(cc.OldY>TopCorner||cc.OldY<=TopCorner-ySize) continue;
                e.Graphics.FillRectangle(Brushes.Orange,(cc.OldX-LeftCorner)*SquareSizeL,
                    (TopCorner-cc.OldY)*SquareSizeL,SquareSizeL,SquareSizeL);
            }
            if(CurrentCells!=null) {
                foreach(ChangedCell cc in CurrentCells) {
                    if(cc.OldX<LeftCorner||cc.OldX>=LeftCorner+xSize) continue;
                    if(cc.OldY>TopCorner||cc.OldY<=TopCorner-ySize) continue;
                    e.Graphics.FillRectangle(Brushes.Green,SquareSizeS+(cc.OldX-LeftCorner)*SquareSizeL,
                        SquareSizeS+(TopCorner-cc.OldY)*SquareSizeL,SquareSizeM,SquareSizeM);
                }
            }
            for(int x=0;x<xSize;x++) {
                e.Graphics.DrawLine(Pens.Black,x*SquareSizeL,0,x*SquareSizeL,ySize*SquareSizeL);
            }
            for(int y=0;y<ySize;y++) {
                e.Graphics.DrawLine(Pens.Black,0,y*SquareSizeL,xSize*SquareSizeL,y*SquareSizeL);
            }
            if(Selected) {
                e.Graphics.DrawRectangle(Pens.Red,(SelectedX-LeftCorner)*SquareSizeL,
                    (TopCorner-SelectedY)*SquareSizeL,SquareSizeL,SquareSizeL);
            }
        }

        public static void SaveMap(string FilePath,int FileType) {
            if(Overlays==0) {
                System.Windows.Forms.MessageBox.Show("You need to overlay some mods to make a map of.", "Error");
                return;
            }
            if(Overlays>=64) {
                System.Windows.Forms.MessageBox.Show("Can only make a map of up to 63 overlayed mods.", "Error");
                return;
            }
            //Find the left,right,top and bottom
            int LeftMost=0,RightMost=0,TopMost=0,BottomMost=0;
            foreach(ChangedCell cc in OverlaidCells) {
                if(cc.OldX>RightMost) RightMost=cc.OldX;
                if(cc.OldX<LeftMost) LeftMost=cc.OldX;
                if(cc.OldY<BottomMost) BottomMost=cc.OldY;
                if(cc.OldY>TopMost) TopMost=cc.OldY;
            }
            BottomMost-=1;
            RightMost+=1;
            //Keep track of how many cells are at each spot
            byte[,] CellTotalCount=new byte[1+RightMost-LeftMost, 1+TopMost-BottomMost];
            byte[,] CellCounted=new byte[1+RightMost-LeftMost, 1+TopMost-BottomMost];
            foreach(ChangedCell cc in OverlaidCells) {
                CellTotalCount[cc.OldX-LeftMost, TopMost-cc.OldY]++;
            }
            Brush[] brushes=new Brush[Overlays];
            byte[] colours=new byte[3];
            byte m,last;
            for(byte i=1;i<=Overlays;i++) {
                m=(byte)(i%3);
                last=(byte)(i%4);
                colours[(m+0)%3]=(byte)(last*64);
                last=(byte)(((i-last)/4)%4);
                colours[(m+1)%3]=(byte)(last*64);
                last=(byte)(((i-last)/4)%4);
                colours[(m+2)%3]=(byte)(last*64);
                colours[1]=(byte)((colours[1]+128)%256);
                brushes[i-1]=new SolidBrush(Color.FromArgb(colours[0],+colours[1],colours[2]));
            }
            //Create an image
            Bitmap bmp=new Bitmap((3+RightMost-LeftMost)*OutSizeL, (3+TopMost-BottomMost)*OutSizeL+10*Overlays);
            Graphics g=Graphics.FromImage(bmp);
            g.Clear(Color.White);
            g.TranslateTransform(OutSizeL, OutSizeM);
            //Draw the cells
            byte BarHeight;
            byte BarOffset;
            foreach(ChangedCell cc in OverlaidCells) {
                BarHeight=(byte)(OutSizeL/CellTotalCount[cc.OldX-LeftMost, TopMost-cc.OldY]);
                BarOffset=(byte)(BarHeight*CellCounted[cc.OldX-LeftMost, TopMost-cc.OldY]++);
                g.FillRectangle(brushes[cc.Overlay], (1+cc.OldX-LeftMost)*OutSizeL,
                    (1+TopMost-cc.OldY)*OutSizeL+BarOffset, OutSizeL, BarHeight);
            }
            //Draw the lines
            for(int x=1;x<(2+RightMost-LeftMost);x++) {
                g.DrawLine(Pens.Black, x*OutSizeL, OutSizeL, x*OutSizeL, (1+TopMost-BottomMost)*OutSizeL);
            }
            for(int y=1;y<(2+TopMost-BottomMost);y++) {
                g.DrawLine(Pens.Black, OutSizeL, y*OutSizeL, (1+RightMost-LeftMost)*OutSizeL, y*OutSizeL);
            }
            //Draw the co-ordinates
            Font font=new Font("arial", 8);
            StringFormat sf=new StringFormat();
            sf.Alignment=StringAlignment.Center;
            sf.LineAlignment=StringAlignment.Center;
            for(int x=LeftMost;x<RightMost;x++) {
                g.DrawString(x.ToString(), font, Brushes.Black, (1+x-LeftMost)*OutSizeL+OutSizeM, OutSizeM,sf);
            }
            for(int y=TopMost;y>BottomMost;y--) {
                g.DrawString(y.ToString(), font, Brushes.Black, 0, OutSizeL-(y-TopMost)*OutSizeL+OutSizeM,sf);
            }
            //Draw the key
            for(int i=0;i<Overlays;i++) {
                g.DrawString((string)OverlayNames[i], font, brushes[i], OutSizeL, (1+TopMost-BottomMost)*OutSizeL+10*i);
            }
            //Save it
            g.Dispose();
            ImageFormat format;
            switch(FileType) {
                case 0: format=ImageFormat.Bmp; break;
                default: format=ImageFormat.Bmp; break;
            }
            bmp.Save("out.bmp",format);
        }
    }
}
